<?php declare(strict_types=1);

namespace App\Form;

use App\Entity\MenuItem;
use App\Repository\MenuItemRepository;
use Doctrine\ORM\QueryBuilder;
use Override;
use Symfony\Bridge\Doctrine\Form\Type\EntityType;
use Symfony\Component\Form\AbstractType;
use Symfony\Component\Form\FormBuilderInterface;
use Symfony\Component\OptionsResolver\OptionsResolver;
use Symfony\Component\Validator\Constraints\Length;
use Symfony\Component\Validator\Constraints\NotBlank;

use function assert;

final class MenuItemType extends AbstractType
{
    #[Override]
    public function buildForm(FormBuilderInterface $builder, array $options): void
    {
        $item = $options['data'];
        assert($item instanceof MenuItem);

        $builder->add('name', \Symfony\Component\Form\Extension\Core\Type\TextType::class, [
            'constraints' => [
                new NotBlank,
                new Length([
                    'min' => 3,
                ]),
            ],
        ]);
        $builder->add('aliases', EntityType::class, [
            'class' => MenuItem::class,
            'choice_label' => 'name',
            'multiple' => true,
            'expanded' => true,
            'query_builder' => static fn(MenuItemRepository $repository): QueryBuilder
                => $repository->getSuitableAliasQueryBuilder($item),
        ]);
    }

    #[Override]
    public function configureOptions(OptionsResolver $resolver): void
    {
        $resolver->setDefaults([
            'data_class' => MenuItem::class,
        ]);
    }
}
